// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef MAILCOMMON_VERSION_H
#define MAILCOMMON_VERSION_H

#define MAILCOMMON_VERSION_STRING "5.22.3"
#define MAILCOMMON_VERSION_MAJOR 5
#define MAILCOMMON_VERSION_MINOR 22
#define MAILCOMMON_VERSION_PATCH 3
#define MAILCOMMON_VERSION ((5<<16)|(22<<8)|(3))

#endif
