/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Command line was: qdbusxml2cpp -m -p pop3settings /usr/obj/ports/mailcommon-22.12.3/build-amd64/src/org.kde.Akonadi.POP3.Settings.xml
 *
 * qdbusxml2cpp is Copyright (C) 2020 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef POP3SETTINGS_H
#define POP3SETTINGS_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.Akonadi.POP3.Settings
 */
class OrgKdeAkonadiPOP3SettingsInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.Akonadi.POP3.Settings"; }

public:
    OrgKdeAkonadiPOP3SettingsInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeAkonadiPOP3SettingsInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<int> authenticationMethod()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("authenticationMethod"), argumentList);
    }

    inline QDBusPendingReply<QStringList> downloadLater()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("downloadLater"), argumentList);
    }

    inline QDBusPendingReply<uint> filterCheckSize()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("filterCheckSize"), argumentList);
    }

    inline QDBusPendingReply<bool> filterOnServer()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("filterOnServer"), argumentList);
    }

    inline QDBusPendingReply<QString> host()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("host"), argumentList);
    }

    inline QDBusPendingReply<bool> intervalCheckEnabled()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("intervalCheckEnabled"), argumentList);
    }

    inline QDBusPendingReply<int> intervalCheckInterval()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("intervalCheckInterval"), argumentList);
    }

    inline QDBusPendingReply<bool> leaveOnServer()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("leaveOnServer"), argumentList);
    }

    inline QDBusPendingReply<int> leaveOnServerCount()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("leaveOnServerCount"), argumentList);
    }

    inline QDBusPendingReply<int> leaveOnServerDays()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("leaveOnServerDays"), argumentList);
    }

    inline QDBusPendingReply<int> leaveOnServerSize()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("leaveOnServerSize"), argumentList);
    }

    inline QDBusPendingReply<QString> login()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("login"), argumentList);
    }

    inline QDBusPendingReply<bool> pipelining()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("pipelining"), argumentList);
    }

    inline QDBusPendingReply<uint> port()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("port"), argumentList);
    }

    inline QDBusPendingReply<QString> precommand()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("precommand"), argumentList);
    }

    inline QDBusPendingReply<> save()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("save"), argumentList);
    }

    inline QDBusPendingReply<QStringList> seenUidList()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("seenUidList"), argumentList);
    }

    inline QDBusPendingReply<QList<int> > seenUidTimeList()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("seenUidTimeList"), argumentList);
    }

    inline QDBusPendingReply<> setAuthenticationMethod(int in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setAuthenticationMethod"), argumentList);
    }

    inline QDBusPendingReply<> setDownloadLater(const QStringList &in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setDownloadLater"), argumentList);
    }

    inline QDBusPendingReply<> setFilterCheckSize(uint in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setFilterCheckSize"), argumentList);
    }

    inline QDBusPendingReply<> setFilterOnServer(bool in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setFilterOnServer"), argumentList);
    }

    inline QDBusPendingReply<> setHost(const QString &in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setHost"), argumentList);
    }

    inline QDBusPendingReply<> setIntervalCheckEnabled(bool in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setIntervalCheckEnabled"), argumentList);
    }

    inline QDBusPendingReply<> setIntervalCheckInterval(int in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setIntervalCheckInterval"), argumentList);
    }

    inline QDBusPendingReply<> setLeaveOnServer(bool in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setLeaveOnServer"), argumentList);
    }

    inline QDBusPendingReply<> setLeaveOnServerCount(int in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setLeaveOnServerCount"), argumentList);
    }

    inline QDBusPendingReply<> setLeaveOnServerDays(int in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setLeaveOnServerDays"), argumentList);
    }

    inline QDBusPendingReply<> setLeaveOnServerSize(int in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setLeaveOnServerSize"), argumentList);
    }

    inline QDBusPendingReply<> setLogin(const QString &in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setLogin"), argumentList);
    }

    inline QDBusPendingReply<> setPipelining(bool in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setPipelining"), argumentList);
    }

    inline QDBusPendingReply<> setPort(uint in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setPort"), argumentList);
    }

    inline QDBusPendingReply<> setPrecommand(const QString &in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setPrecommand"), argumentList);
    }

    inline QDBusPendingReply<> setSeenUidList(const QStringList &in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setSeenUidList"), argumentList);
    }

    inline QDBusPendingReply<> setSeenUidTimeList(const QList<int> &in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setSeenUidTimeList"), argumentList);
    }

    inline QDBusPendingReply<> setStorePassword(bool in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setStorePassword"), argumentList);
    }

    inline QDBusPendingReply<> setTargetCollection(qlonglong in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setTargetCollection"), argumentList);
    }

    inline QDBusPendingReply<> setUnitTestPassword(const QString &in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setUnitTestPassword"), argumentList);
    }

    inline QDBusPendingReply<> setUseSSL(bool in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setUseSSL"), argumentList);
    }

    inline QDBusPendingReply<> setUseTLS(bool in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("setUseTLS"), argumentList);
    }

    inline QDBusPendingReply<bool> storePassword()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("storePassword"), argumentList);
    }

    inline QDBusPendingReply<qlonglong> targetCollection()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("targetCollection"), argumentList);
    }

    inline QDBusPendingReply<QString> unitTestPassword()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("unitTestPassword"), argumentList);
    }

    inline QDBusPendingReply<bool> useSSL()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("useSSL"), argumentList);
    }

    inline QDBusPendingReply<bool> useTLS()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("useTLS"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace kde {
    namespace Akonadi {
      namespace POP3 {
        typedef ::OrgKdeAkonadiPOP3SettingsInterface Settings;
      }
    }
  }
}
#endif
